@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2015, Arvid Norberg
+Copyright (c) 2015-2021, Arvid Norberg
+Copyright (c) 2019, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -43,17 +44,17 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #if TORRENT_USE_INVARIANT_CHECKS
 #include "libtorrent/bitfield.hpp"
-#include "libtorrent/invariant_check.hpp"
+#include "libtorrent/aux_/invariant_check.hpp"
 #endif
 
 #if TORRENT_USE_INVARIANT_CHECKS
-#include "libtorrent/invariant_check.hpp"
+#include "libtorrent/aux_/invariant_check.hpp"
 #include "libtorrent/bitfield.hpp"
 #endif
 
 namespace libtorrent {
 
-class piece_picker;
+struct piece_picker;
 class file_storage;
 
 namespace aux {
@@ -67,6 +68,11 @@ namespace aux {
 
 		void export_progress(vector<std::int64_t, file_index_t> &fp);
 
+		std::int64_t total_on_disk() const
+		{
+			return m_total_on_disk;
+		}
+
 		bool empty() const { return m_file_progress.empty(); }
 		void clear();
 
@@ -75,6 +81,9 @@ namespace aux {
 
 	private:
 
+		// the total number of bytes downloaded to non-pad files
+		std::int64_t m_total_on_disk = 0;
+
 		// this vector contains the number of bytes completely
 		// downloaded (as in passed-hash-check) in each file.
 		// this lets us trigger on individual files completing
@@ -83,7 +92,7 @@ namespace aux {
 		vector<std::int64_t, file_index_t> m_file_progress;
 
 #if TORRENT_USE_INVARIANT_CHECKS
-		friend class libtorrent::invariant_access;
+		friend struct libtorrent::invariant_access;
 		void check_invariant() const;
 
 		// this is used to assert we never add the same piece twice
@@ -92,6 +101,7 @@ namespace aux {
 		// to make sure we never say we've downloaded more bytes of a file than
 		// its file size
 		vector<std::int64_t, file_index_t> m_file_sizes;
+		vector<bool, file_index_t> m_pad_file;
 #endif
 	};
 } }
