@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2003-2018, Arvid Norberg
+Copyright (c) 2003-2009, 2013-2020, Arvid Norberg
+Copyright (c) 2016-2017, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -75,53 +76,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/span.hpp"
 #include "libtorrent/string_view.hpp"
 #include "libtorrent/aux_/aligned_union.hpp"
+#include "libtorrent/aux_/strview_less.hpp"
 
 namespace libtorrent {
 
 #if TORRENT_ABI_VERSION == 1
-	struct lazy_entry;
 	// backwards compatibility
 	using type_error = system_error;
 #endif
 	struct bdecode_node;
 
-namespace aux {
-
-#if !defined TORRENT_CXX11_ABI && (__cplusplus > 201103) || (defined _MSC_VER && _MSC_VER >= 1900)
-		// this enables us to compare a string_view against the std::string that's
-		// held by the std::map
-		// is_transparent was introduced in C++14
-		struct strview_less
-		{
-			using is_transparent = std::true_type;
-			template <typename T1, typename T2>
-			bool operator()(T1 const& rhs, T2 const& lhs) const
-			{ return rhs < lhs; }
-		};
-
-		template<class T> using map_string = std::map<std::string, T, aux::strview_less>;
-#else
-		template<class T>
-		struct map_string : std::map<std::string, T>
-		{
-			using base = std::map<std::string, T>;
-			using base::base;
-			map_string() = default;
-			map_string(base&& rhs) : base(std::move(rhs)) {}
-
-			typename base::iterator find(const string_view& key)
-			{
-				return this->base::find(key.to_string());
-			}
-
-			typename base::const_iterator find(const string_view& key) const
-			{
-				return this->base::find(key.to_string());
-			}
-		};
-#endif
-	}
-
 	// The ``entry`` class represents one node in a bencoded hierarchy. It works as a
 	// variant type, it can be either a list, a dictionary (``std::map``), an integer
 	// or a string.
@@ -132,7 +96,7 @@ namespace aux {
 		// the key is always a string. If a generic entry would be allowed
 		// as a key, sorting would become a problem (e.g. to compare a string
 		// to a list). The definition doesn't mention such a limit though.
-		using dictionary_type = aux::map_string<entry>;
+		using dictionary_type = std::map<std::string, entry, aux::strview_less>;
 		using string_type = std::string;
 		using list_type = std::vector<entry>;
 		using integer_type = std::int64_t;
@@ -157,6 +121,11 @@ namespace aux {
 		// newly constructed entry
 		entry(dictionary_type); // NOLINT
 		entry(span<char const>); // NOLINT
+		entry(list_type); // NOLINT
+		entry(integer_type); // NOLINT
+		entry(preformatted_type); // NOLINT
+
+		// hidden
 		template <typename U, typename Cond = typename std::enable_if<
 			std::is_same<U, entry::string_type>::value
 			|| std::is_same<U, string_view>::value
@@ -170,9 +139,6 @@ namespace aux {
 			new(&data) string_type(std::move(v));
 			m_type = string_t;
 		}
-		entry(list_type); // NOLINT
-		entry(integer_type); // NOLINT
-		entry(preformatted_type); // NOLINT
 
 		// construct an empty entry of the specified type.
 		// see data_type enum.
@@ -193,14 +159,16 @@ namespace aux {
 
 		// copies the structure of the right hand side into this
 		// entry.
-#if TORRENT_ABI_VERSION == 1
-		entry& operator=(lazy_entry const&) &;
-#endif
 		entry& operator=(bdecode_node const&) &;
 		entry& operator=(entry const&) &;
 		entry& operator=(entry&&) & noexcept;
 		entry& operator=(dictionary_type) &;
 		entry& operator=(span<char const>) &;
+		entry& operator=(list_type) &;
+		entry& operator=(integer_type) &;
+		entry& operator=(preformatted_type) &;
+
+		// hidden
 		template <typename U, typename Cond = typename std::enable_if<
 			std::is_same<U, entry::string_type>::value
 			|| std::is_same<U, char const*>::value>::type>
@@ -214,9 +182,6 @@ namespace aux {
 #endif
 			return *this;
 		}
-		entry& operator=(list_type) &;
-		entry& operator=(integer_type) &;
-		entry& operator=(preformatted_type) &;
 
 		// The ``integer()``, ``string()``, ``list()`` and ``dict()`` functions
 		// are accessors that return the respective type. If the ``entry`` object
@@ -266,15 +231,15 @@ namespace aux {
 		// To make it easier to extract information from a torrent file, the
 		// class torrent_info exists.
 		integer_type& integer();
-		const integer_type& integer() const;
+		integer_type const& integer() const;
 		string_type& string();
-		const string_type& string() const;
+		string_type const& string() const;
 		list_type& list();
-		const list_type& list() const;
+		list_type const& list() const;
 		dictionary_type& dict();
-		const dictionary_type& dict() const;
+		dictionary_type const& dict() const;
 		preformatted_type& preformatted();
-		const preformatted_type& preformatted() const;
+		preformatted_type const& preformatted() const;
 
 		// swaps the content of *this* with ``e``.
 		void swap(entry& e);
@@ -291,7 +256,7 @@ namespace aux {
 		// existing element at the given key. If the key is not found, it will
 		// throw ``system_error``.
 		entry& operator[](string_view key);
-		const entry& operator[](string_view key) const;
+		entry const& operator[](string_view key) const;
 
 		// These functions requires the entry to be a dictionary, if it isn't
 		// they will throw ``system_error``.
@@ -336,6 +301,7 @@ namespace aux {
 		std::uint8_t m_type:7;
 
 	public:
+		// hidden
 		// in debug mode this is set to false by bdecode to indicate that the
 		// program has not yet queried the type of this entry, and should not
 		// assume that it has a certain type. This is asserted in the accessor
@@ -347,10 +313,10 @@ namespace aux {
 	TORRENT_EXPORT bool operator==(entry const& lhs, entry const& rhs);
 	inline bool operator!=(entry const& lhs, entry const& rhs) { return !(lhs == rhs); }
 
-namespace detail {
+namespace aux {
 
 	// internal
-	TORRENT_EXPORT string_view integer_to_str(span<char> buf
+	TORRENT_EXPORT string_view integer_to_str(std::array<char, 21>& buf
 		, entry::integer_type val);
 }
 
