@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2012-2018, Arvid Norberg
+Copyright (c) 2015, 2017-2020, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -57,8 +57,8 @@ namespace libtorrent {
 		// session_stats_alert object.
 		int value_index;
 #if TORRENT_ABI_VERSION == 1
-		static constexpr metric_type_t TORRENT_DEPRECATED_MEMBER type_counter = metric_type_t::counter;
-		static constexpr metric_type_t TORRENT_DEPRECATED_MEMBER type_gauge = metric_type_t::gauge;
+		TORRENT_DEPRECATED static constexpr metric_type_t type_counter = metric_type_t::counter;
+		TORRENT_DEPRECATED static constexpr metric_type_t type_gauge = metric_type_t::gauge;
 #endif
 		metric_type_t type;
 	};
